/*
Copyright 2023 Matt F

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include QMK_KEYBOARD_H

#define FN        MO(1)
#define EXTRA     TG(2)
#define BACKFN    LT(1, KC_BSPC)


const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

    [0] = LAYOUT(
	KC_ESC,  KC_GRV,  KC_1,    KC_2,    KC_3,    KC_4,    KC_5,    KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS, KC_EQL,  KC_BSPC, KC_BSPC,  \
		     KC_TAB,  KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,    KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_LBRC, KC_RBRC, KC_DEL,            \
		     BACKFN,  KC_A,    KC_S,    KC_D,    KC_F,    KC_G,    KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT, KC_ENT,                     \
		     SC_LSPO, KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,    KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, SC_RSPC,          KC_UP,   FN,       \
	KC_LCTL, KC_LGUI, KC_LALT,                   FN,      KC_SPC,  KC_SPC,                    KC_RALT, KC_RCTL, KC_LGUI, KC_LEFT, KC_DOWN, KC_RGHT    \
	),

/* Default with Cadet shifts and no Caps Lock
 *  .-----------------------------------------------------------------------------------------------------------------------------------------------+
 *  | ESC    | ~      | 1      | 2      | 3      | 4      | 5      | 6      | 7      | 8      | 9      |  0     | -      | +      | Backspc| Back   |
 *  |        | `      |        |        |        |        |        |        |        |        |        |        | _      | =      |        |        |  
 *  +--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+
 *       |    TAB     | Q      | W      | E      | R      | T      | Y      | U      | I      | O      | P      | [      | ]      |  DELETE     |
 *       |            |        |        |        |        |        |        |        |        |        |        | {      | }      |             |  
 *    +---------------+--------+--------+--------+--------+------- +--------+--------+--------+--------+--------+--------|--------+-------------+
 *    | BACKSPACE (FN)| A      | S      | D      | F      | G      | H      | J      | K      | L      | ;      | '      |  ENTER            |
 *    |               |        |        |        |        |        |        |        |        |        | :      | "      |                   |  
 *  +-----------------+--------+--------+--------+--------+--------+--------+--------+--------------------------+--------+-------+-----------+-----+
 * |  CADET SHIFT (   | Z      | X      | C      | V      | B      | N      | M      | <      | >      | /      | CADET SHIFT  ) |  UP    |  Fn    |
 * |                  |        |        |        |        |        |        |        | ,      | .      | ?      |                |        |        |
 * |----------+----------+----------+--------------------------------------------------------+--------+--------+--------+--------+--------|--------+
 * | LCTRL    | LGUI     | LALT     |         Fn        |   SPACE     |        Space         | RALT   | GUI    | RCTRL  | LEFT   | DOWN   | RIGHT  |
 * |          |          |          |                   |             |                      |        |        |        |        |        |        |  
 * '--------------------------------------------------------------------------------------------------------------------------------------+--------+
*/


    [1] = LAYOUT(
    _______, _______, KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10,  KC_F11,  KC_F12,  KC_DEL,  QK_BOOT, \
             _______, KC_HOME, KC_UP,   KC_END,  KC_PGUP, _______, _______, _______, KC_INS,  _______, KC_PSCR, _______, _______, KC_BSLS,          \
             _______, KC_LEFT, KC_DOWN, KC_RGHT, KC_PGDN, _______, _______, KC_MPRV, KC_MPLY, KC_MNXT, _______, _______, RGB_TOG,                   \
             _______, RGB_M_R, RGB_M_X, RGB_M_SW,RGB_M_K, RGB_M_B, RGB_M_G, KC_MUTE, KC_VOLD, KC_VOLU, KC_BSLS, KC_CAPS,          KC_PGUP, _______, \
    _______, QK_BOOT, _______,                   _______, KC_ENT,  EXTRA,                     _______, _______, _______, KC_HOME, KC_PGDN, KC_END   \
    ), 

/* Fn
 *  .-----------------------------------------------------------------------------------------------------------------------------------------------+
 *  |        |        | F1     | F2     | F3     | F4     | F5     | F6     | F7     | F8     | F9     |  F10   | F11    | F12    | DELETE | RESET  |
 *  |        |        |        |        |        |        |        |        |        |        |        |        |        |        |        |        |  
 *  +--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+
 *       |            | HOME   | UP     | END    | PGUP   |        |        |        | INS    |        | Print  |        |        |  PIPE \     |
 *       |            |        |        |        |        |        |        |        |        |        | Screen |        |        |             |  
 *    +---------------+--------+--------+--------+--------+------- +--------+--------+--------+--------+--------+--------|--------+-------------+
 *    |               | LEFT   | DOWN   | RIGHT  | PGDN   |        |        | PREV   | PLAY   | NEXT   |        |        |   RGB TOG          |
 *    |               |        |        |        |        |        |        |        |        |        |        |        |                    |  
 *  +-----------------+--------+--------+--------+--------+--------+--------+--------+--------------------------+--------+-------+------------+----+
 * |                  | Rainbow| Xmas   | Swirl  | Knight |Breathe |GradieNt| MUTE   | VOL DN | VOL UP | PIPE \ | CAPS LOCK      | PAGEUP |  Fn    |
 * |                  |        |        |        |        |        |        |        |        |        |        |                |        |        |
 * |----------+----------+----------+---------------------------------------------------------+--------+--------+----------------+--------+--------+
 * |          |  RESET   |          |         Fn        |   ENTER     |     EXTRA            |        |        |        | HOME   | PAGEDN | END    |
 * |          |          |          |                   |             |                      |        |        |        |        |        |        |  
 * '--------------------------------------------------------------------------------------------------------------------------------------+--------+
 */

    [2] = LAYOUT(
    QK_BOOT, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, KC_PMNS, KC_PPLS, KC_BSPC, _______, \
             _______, _______, _______, _______, _______, _______, _______, _______, KC_P7,   KC_P8,   KC_P9,   _______, _______, KC_DEL,           \
             _______, _______, _______, _______, _______, _______, _______, _______, KC_P4,   KC_P5,   KC_P6,   _______, KC_PENT,                   \
             _______, _______, _______, _______, _______, _______, _______, _______, KC_P1,   KC_P2,   KC_P3,   KC_PPLS,          _______, _______, \
    _______, _______, _______,                   EXTRA,   KC_TAB,  KC_P0,                     KC_PDOT, _______, _______, _______, _______, KC_TRNS  \
    ), 

/* EXTR
 *  .-----------------------------------------------------------------------------------------------------------------------------------------------+
 *  |        |        |        |        |        |        |        |        |        |        |        |        | -      | +      | BACKSP |        |
 *  |        |        |        |        |        |        |        |        |        |        |        |        |        |        |        |        |  
 *  +--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+
 *       |            |        |        |        |        |        |        |        | 7      | 8      | 9      |        |        | DELETE      |
 *       |            |        |        |        |        |        |        |        |        |        |        |        |        |             |  
 *    +---------------+--------+--------+--------+--------+------- +--------+--------+--------+--------+--------+--------|--------+-------------+ 
 *    |               |        |        |        |        |        |        |        | 4      | 5      | 6      |        |  ENT               |
 *    |               |        |        |        |        |        |        |        |        |        |        |        |                    |  
 *  +-----------------+--------+--------+--------+--------+--------+--------+--------+--------------------------+--------+-------+------------+----+
 * |                  |        |        |        |        |        |        |        | 1      | 2      | 3      |  +             |        |  TRNS  |
 * |                  |        |        |        |        |        |        |        |        |        |        |                |        |        |
 * |----------+----------+----------+---------------------------------------------------------+--------+--------+----------------+--------+--------+
 * |          |          |          |        EXTRA      |   ENTER     |                0     |  .     |        |        |        |        |        |
 * |          |          |          |                   |             |                      |        |        |        |        |        |        |  
 * '--------------------------------------------------------------------------------------------------------------------------------------+--------+
 */
};

#if defined(ENCODER_MAP_ENABLE)
const uint16_t PROGMEM encoder_map[][NUM_ENCODERS][2] = {
    [0] = { ENCODER_CCW_CW(KC_MS_WH_UP, KC_MS_WH_DOWN), ENCODER_CCW_CW(KC_MS_WH_UP, KC_MS_WH_DOWN),   ENCODER_CCW_CW(KC_VOLD, KC_VOLU)  },
    [1] = { ENCODER_CCW_CW(C(KC_TAB), C(S(KC_TAB))),    ENCODER_CCW_CW(KC_MS_WH_UP, KC_MS_WH_DOWN),   ENCODER_CCW_CW(RGB_SAD, RGB_SAI)  },
    [2] = { ENCODER_CCW_CW(KC_VOLD, KC_VOLU),           ENCODER_CCW_CW(KC_RIGHT, KC_LEFT),            ENCODER_CCW_CW(RGB_SPD, RGB_SPI)  },
};
#endif



    /* Encoder Positions
 *  .-----------------------------------------------------------------------------------------------------------------------------------------------+
 *  |        |        |        |        |        |        |        |        |        |        |        |        |        |        |        |        |
 *  |     E1 |        |        |        |        |        |        |        |        |        |        |        |        |        |        |        |  
 *  +--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+
 *       |            |        |        |        |        |        |        |        |        |        |        |        |        |             |
 *       |            |        |        |        |        |        |        |        |        |        |        |        |        |             |  
 *    +---------------+--------+--------+--------+--------+------- +--------+--------+--------+--------+--------+--------|--------+-------------+
 *    |               |        |        |        |        |        |        |        |        |        |        |        |                    |
 *    |               |        |        |        |        |        |        |        |        |        |        |        |                    |  
 *  +-----------------+--------+--------+--------+--------+--------+--------+--------+--------------------------+--------+-------+------------+----+
 * |                  |        |        |        |        |        |        |        |        |        |        |                |        |        |
 * |                  |        |        |        |        |        |        |        |        |        |        |                |        |     E2 |
 * |----------+----------+----------+---------------------------------------------------------+--------+--------+----------------+--------+--------+
 * |          |          |          |                   |             |                      |        |        |        |        |        |        |
 * |       E3 |          |          |                   |         E3  |                      |        |        |     E2 |        |        |        |  
 * '--------------------------------------------------------------------------------------------------------------------------------------+--------+
  */


